﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;

/*
 * USB-ISS example.
 * Sets the USB-ISS to I2C mode and then reads the light and range values from an SRF08.
 */

namespace usb_iss
{
    public partial class Form1 : Form
    {
        static SerialPort USB_PORT;
        byte[] SerBuf = new byte[64];
        byte usb_found;

        public Form1()
        {
            InitializeComponent();
            USB_PORT = new SerialPort();
        }

        private void Form1_load(object sender, EventArgs e)
        {

        }

        private void transmit(byte write_bytes)
        {
            try
            {
                if (usb_found == 1) USB_PORT.Write(SerBuf, 0, write_bytes);      // writes specified amount of SerBuf out on COM port
            }
            catch (Exception)
            {
                usb_found = 0;
                MessageBox.Show("write fail");
                return;
            }
        }

        private void receive(byte read_bytes)
        {
            byte x;
            for (x = 0; x < read_bytes; x++)       // this will call the read function for the passed number times, 
            {                                      // this way it ensures each byte has been correctly recieved while
                try                                // still using timeouts
                {
                    if (usb_found == 1) USB_PORT.Read(SerBuf, x, 1);     // retrieves 1 byte at a time and places in SerBuf at position x
                }
                catch (Exception)                   // timeout or other error occured, set lost comms indicator
                {
                    usb_found = 0;
                    MessageBox.Show("read fail");
                    return;
                }
            }
        }

        private void comboBox_comport_MouseDown(object sender, MouseEventArgs e)
        {
            // Grab the list of available COM ports and place them in the combo box
            comboBox_comport.Items.Clear();
            string[] comportnames = new string[100];
            comportnames = SerialPort.GetPortNames();
            var sortedList = comportnames.OrderBy(port => Convert.ToInt32(port.Replace("COM", string.Empty)));
            foreach (string port in sortedList)
            {
                comboBox_comport.Items.Add(port);                      // places each "COMx" name into combobox
            }
        }

        private void comboBox_comport_SelectedIndexChanged(object sender, EventArgs e)
        {
            timer1.Stop();
            try
            {
                if (USB_PORT.IsOpen) USB_PORT.Close(); // close any existing handle
            }
            catch
            {
                MessageBox.Show("Error closing to COM port", "USB-ISS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                Application.Exit();
            }
            try
            {
                USB_PORT.PortName = comboBox_comport.Text;    // retrieves "COMx" from selection in combo box
                USB_PORT.ReadTimeout = 500;
                USB_PORT.WriteTimeout = 500;
                USB_PORT.Open();
                usb_found = 1;

                setup_iss();

                timer1.Enabled = true;
                timer1.Start();

            }
            catch
            {
                MessageBox.Show("Error opening to COM port", "USB-ISS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                usb_found = 0;
                timer1.Stop();
            }
        }

        private void setup_iss()
        {
            // Set the USB-ISS to I2C mode
            SerBuf[0] = 0x5A;       // USB-ISS command
            SerBuf[1] = 0x02;       // USB-ISS mode command
            SerBuf[2] = 0x60;       // 100KHz Hardware I2C port
            transmit(3);
            receive(2);             // Get the response bytes
            if (SerBuf[0] == 0)     // If we receive a 0 then there has been a problem
            {
                MessageBox.Show("Error setting I2C mode", "USB-ISS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                usb_found = 0;
            }

            SerBuf[0] = 0x5A;       // USB-ISS command
            SerBuf[1] = 0x01;       // USB-ISS version command
            transmit(2);
            receive(3);

            label_id.Text = "USB-ISS id : " + SerBuf[0].ToString();
            label_issver.Text = "USB-ISS ver : " + SerBuf[1].ToString();
            label_mode.Text = "USB-ISS mode : " + SerBuf[2].ToString();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (usb_found == 1)
            {
                // Start a ranging from the SRF08
                SerBuf[0] = 0x55;
                SerBuf[1] = 0xE0;
                SerBuf[2] = 0x00;
                SerBuf[3] = 0x01;
                SerBuf[4] = 0x51;
                transmit(5);
                receive(1);

                if (SerBuf[0] == 0)     // If we receive a 0 then there has been a problem writing to the I2C device
                {
                    MessageBox.Show("Error writing to I2C device", "USB-ISS", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    usb_found = 0;
                }

                System.Threading.Thread.Sleep(100); // Wait for the ranging to happen

                // Read back the SRF08 software value, Light value and range from the SRF08
                SerBuf[0] = 0x55;
                SerBuf[1] = 0xE1;
                SerBuf[2] = 0x00;
                SerBuf[3] = 0x04;
                transmit(4);
                receive(4);

                // Calcualte the range
                int range = SerBuf[2];
                range <<= 8;
                range += SerBuf[3];

                label_ver.Text = "SRF08 ver : " + SerBuf[0].ToString();
                label_light.Text = "Light : " + SerBuf[1].ToString();
                label_range.Text = "Range : " + range.ToString();
            }
        }
    }
}
